@@ -1,4 +1,5 @@
 import logging
+from typing import Dict
 from absl import logging as absl_logging
 from .singleton import Singleton
 absl_logging._warn_preinit_stderr = 0
@@ -10,14 +11,14 @@ class Logger(Singleton):
     VERBOSE = True
     QUIET = False
 
-    def __init__(self, output_log="output.log"):
-        self.__loggers = {}
+    def __init__(self, output_log: str = "output.log"):
+        self.__loggers: Dict[str, logging.Logger] = {}
         self.__output_log = output_log
 
-    def get_logger(self, name):
+    def get_logger(self, name: str) -> logging.Logger:
 
-        if self.__loggers.get(name):
-            return self.__loggers.get(name)
+        if self.__loggers.get(name) is not None:
+            return self.__loggers.get(name)  # type: ignore
         else:
             logger = logging.getLogger(name)
             logger.setLevel(logging.INFO)
