@@ -5,7 +5,7 @@ usage: subaligner_1pass [-h] -v VIDEO_PATH -s SUBTITLE_PATH [-l MAX_LOGLOSS] [-t
 Run single-stage alignment
 
 optional arguments:
-  -h, --help            Show this help message and exit
+  -h, --help            show this help message and exit
   -l MAX_LOGLOSS, --max_logloss MAX_LOGLOSS
                         Max global log loss for alignment
   -tod TRAINING_OUTPUT_DIRECTORY, --training_output_directory TRAINING_OUTPUT_DIRECTORY
@@ -26,6 +26,7 @@ import argparse
 import sys
 import traceback
 import os
+import tempfile
 
 
 def main():
@@ -38,7 +39,7 @@ def main():
         print("Subaligner is not installed")
         sys.exit(20)
 
-    parser = argparse.ArgumentParser(description="Run single-stage alignment")
+    parser = argparse.ArgumentParser(description="Run single-stage alignment", formatter_class=argparse.RawTextHelpFormatter)
     required_args = parser.add_argument_group("required arguments")
     required_args.add_argument(
         "-v",
@@ -89,6 +90,12 @@ def main():
     if FLAGS.subtitle_path == "":
         print("--subtitle_path was not passed in")
         sys.exit(21)
+    if FLAGS.subtitle_path.lower().startswith("http") and FLAGS.output == "":
+        print("--output was not passed in for alignment on a remote subtitle file")
+        sys.exit(21)
+
+    local_video_path = FLAGS.video_path
+    local_subtitle_path = FLAGS.subtitle_path
 
     from subaligner.logger import Logger
     Logger.VERBOSE = FLAGS.debug
@@ -97,12 +104,25 @@ def main():
     from subaligner.subtitle import Subtitle
     from subaligner.exception import UnsupportedFormatException
     from subaligner.exception import TerminalException
+    from subaligner.utils import Utils
 
     try:
+        if FLAGS.video_path.lower().startswith("http"):
+            _, local_video_path = tempfile.mkstemp()
+            _, video_file_extension = os.path.splitext(FLAGS.video_path.lower())
+            local_video_path = "{}{}".format(local_video_path, video_file_extension)
+            Utils.download_file(FLAGS.video_path, local_video_path)
+
+        if FLAGS.subtitle_path.lower().startswith("http"):
+            _, local_subtitle_path = tempfile.mkstemp()
+            _, subtitle_file_extension = os.path.splitext(FLAGS.subtitle_path.lower())
+            local_subtitle_path = "{}{}".format(local_subtitle_path, subtitle_file_extension)
+            Utils.download_file(FLAGS.subtitle_path, local_subtitle_path)
+
         predictor = Predictor()
         subs, audio_file_path, voice_probabilities, frame_rate = predictor.predict_single_pass(
-            video_file_path=FLAGS.video_path,
-            subtitle_file_path=FLAGS.subtitle_path,
+            video_file_path=local_video_path,
+            subtitle_file_path=local_subtitle_path,
             weights_dir=os.path.join(FLAGS.training_output_directory, "models/training/weights")
         )
 
@@ -115,24 +135,36 @@ def main():
             print(
                 "Alignment failed with a too high loss value: {}".format(log_loss)
             )
-            exit(22)
+            _remove_tmp_files(FLAGS, local_video_path, local_subtitle_path)
+            sys.exit(22)
     except UnsupportedFormatException as e:
         print(
             "{}\n{}".format(str(e), "".join(traceback.format_stack()) if FLAGS.debug else "")
         )
+        _remove_tmp_files(FLAGS, local_video_path, local_subtitle_path)
         sys.exit(23)
     except TerminalException as e:
         print(
             "{}\n{}".format(str(e), "".join(traceback.format_stack()) if FLAGS.debug else "")
         )
+        _remove_tmp_files(FLAGS, local_video_path, local_subtitle_path)
         sys.exit(24)
     except Exception as e:
         print(
             "{}\n{}".format(str(e), "".join(traceback.format_stack()) if FLAGS.debug else "")
         )
+        _remove_tmp_files(FLAGS, local_video_path, local_subtitle_path)
         sys.exit(1)
     else:
-        exit(0)
+        _remove_tmp_files(FLAGS, local_video_path, local_subtitle_path)
+        sys.exit(0)
+
+
+def _remove_tmp_files(flags, local_video_path, local_subtitle_path):
+    if flags.video_path.lower().startswith("http") and os.path.exists(local_video_path):
+        os.remove(local_video_path)
+    if flags.subtitle_path.lower().startswith("http") and os.path.exists(local_subtitle_path):
+        os.remove(local_subtitle_path)
 
 
 if __name__ == "__main__":
