@@ -1,5 +1,10 @@
 import os
 import subprocess
+import pysubs2
+import requests
+import shutil
+
+from typing import Optional, TextIO, BinaryIO, Union
 from pycaption import (
     CaptionConverter,
     SRTReader,
@@ -7,7 +12,6 @@ from pycaption import (
     DFXPReader,
     SRTWriter,
 )
-import pysubs2
 from .exception import TerminalException
 
 
@@ -16,7 +20,7 @@ class Utils(object):
     """
 
     @staticmethod
-    def srt2ttml(srt_file_path, ttml_file_path=None):
+    def srt2ttml(srt_file_path: str, ttml_file_path: Optional[str] = None) -> None:
         """Convert SubRip subtitles to TTML subtitles.
 
         Arguments:
@@ -24,6 +28,7 @@ class Utils(object):
             ttml_file_path {string} -- The path to the TTML file.
         """
 
+        file: Union[TextIO, BinaryIO]
         converter = CaptionConverter()
         with open(srt_file_path, "r", encoding="utf8") as file:
             converter.read(file.read(), SRTReader())
@@ -33,7 +38,7 @@ class Utils(object):
             file.write(converter.write(DFXPWriter()).encode("utf-8"))
 
     @staticmethod
-    def ttml2srt(ttml_file_path, srt_file_path=None):
+    def ttml2srt(ttml_file_path: str, srt_file_path: Optional[str] = None) -> None:
         """Convert TTML subtitles to SubRip subtitles.
 
         Arguments:
@@ -41,6 +46,7 @@ class Utils(object):
             srt_file_path {string} -- The path to the SubRip file.
         """
 
+        file: Union[TextIO, BinaryIO]
         converter = CaptionConverter()
         with open(ttml_file_path, "r", encoding="utf8") as file:
             converter.read(file.read(), DFXPReader())
@@ -50,16 +56,16 @@ class Utils(object):
             file.write(converter.write(SRTWriter()).encode("utf-8"))
 
     @staticmethod
-    def srt2vtt(srt_file_path, vtt_file_path=None, timeout_secs=30):
+    def srt2vtt(srt_file_path: str, vtt_file_path: Optional[str] = None, timeout_secs: int = 30) -> None:
         """Convert SubRip subtitles to WebVTT subtitles.
 
         Arguments:
             srt_file_path {string} -- The path to the SubRip file.
             vtt_file_path {string} -- The path to the WebVTT file.
         """
-        if vtt_file_path is None:
-            vtt_file_path = srt_file_path.replace(".srt", ".vtt")
-        command = "ffmpeg -y -i {0} -f webvtt {1}".format(srt_file_path, vtt_file_path)
+
+        _vtt_file_path = srt_file_path.replace(".srt", ".vtt") if vtt_file_path is None else vtt_file_path
+        command = "ffmpeg -y -i {0} -f webvtt {1}".format(srt_file_path, _vtt_file_path)
         with subprocess.Popen(
             command.split(),
             shell=False,
@@ -77,7 +83,7 @@ class Utils(object):
                             srt_file_path, std_err
                         )
                     )
-                Utils.remove_trailing_newlines(vtt_file_path)
+                Utils.remove_trailing_newlines(_vtt_file_path)
             except subprocess.TimeoutExpired as te:
                 process.kill()
                 raise TerminalException(
@@ -99,16 +105,16 @@ class Utils(object):
                 os.system("stty sane")
 
     @staticmethod
-    def vtt2srt(vtt_file_path, srt_file_path=None, timeout_secs=30):
+    def vtt2srt(vtt_file_path: str, srt_file_path: Optional[str] = None, timeout_secs: int = 30) -> None:
         """Convert WebVTT subtitles to SubRip subtitles.
 
         Arguments:
             vtt_file_path {string} -- The path to the WebVTT file.
             srt_file_path {string} -- The path to the SubRip file.
         """
-        if srt_file_path is None:
-            srt_file_path = vtt_file_path.replace(".vtt", ".srt")
-        command = "ffmpeg -y -i {0} -f srt {1}".format(vtt_file_path, srt_file_path)
+
+        _srt_file_path = vtt_file_path.replace(".vtt", ".srt") if srt_file_path is None else srt_file_path
+        command = "ffmpeg -y -i {0} -f srt {1}".format(vtt_file_path, _srt_file_path)
         with subprocess.Popen(
             command.split(),
             shell=False,
@@ -126,7 +132,7 @@ class Utils(object):
                             vtt_file_path, std_err
                         )
                     )
-                Utils.remove_trailing_newlines(srt_file_path)
+                Utils.remove_trailing_newlines(_srt_file_path)
             except subprocess.TimeoutExpired as te:
                 process.kill()
                 process.wait()
@@ -150,7 +156,7 @@ class Utils(object):
                 os.system("stty sane")
 
     @staticmethod
-    def srt2ass(srt_file_path, ass_file_path=None):
+    def srt2ass(srt_file_path: str, ass_file_path: Optional[str] = None) -> None:
         """Convert SubRip subtitles to Advanced SubStation Alpha v4.0+ subtitles.
 
         Arguments:
@@ -158,11 +164,11 @@ class Utils(object):
             ass_file_path {string} -- The path to the ASS file.
         """
 
-        Utils.__convert_subtitle(srt_file_path, "srt", ass_file_path, "ass", "ass")
-        Utils.remove_trailing_newlines(ass_file_path)
+        new_ass_file_path = Utils.__convert_subtitle(srt_file_path, "srt", ass_file_path, "ass", "ass")
+        Utils.remove_trailing_newlines(new_ass_file_path)
 
     @staticmethod
-    def ass2srt(ass_file_path, srt_file_path=None):
+    def ass2srt(ass_file_path: str, srt_file_path: Optional[str] = None) -> None:
         """Convert Advanced SubStation Alpha v4.0+ subtitles to SubRip subtitles.
 
         Arguments:
@@ -170,11 +176,11 @@ class Utils(object):
             srt_file_path {string} -- The path to the SubRip file.
         """
 
-        Utils.__convert_subtitle(ass_file_path, "ass", srt_file_path, "srt", "srt")
-        Utils.remove_trailing_newlines(srt_file_path)
+        new_srt_file_path = Utils.__convert_subtitle(ass_file_path, "ass", srt_file_path, "srt", "srt")
+        Utils.remove_trailing_newlines(new_srt_file_path)
 
     @staticmethod
-    def srt2ssa(srt_file_path, ssa_file_path=None):
+    def srt2ssa(srt_file_path: str, ssa_file_path: Optional[str] = None) -> None:
         """Convert SubRip subtitles to SubStation Alpha v4.0 subtitles.
 
         Arguments:
@@ -182,11 +188,11 @@ class Utils(object):
             ssa_file_path {string} -- The path to the SSA file.
         """
 
-        Utils.__convert_subtitle(srt_file_path, "srt", ssa_file_path, "ssa", "ssa")
-        Utils.remove_trailing_newlines(ssa_file_path)
+        new_ssa_file_path = Utils.__convert_subtitle(srt_file_path, "srt", ssa_file_path, "ssa", "ssa")
+        Utils.remove_trailing_newlines(new_ssa_file_path)
 
     @staticmethod
-    def ssa2srt(ssa_file_path, srt_file_path=None):
+    def ssa2srt(ssa_file_path: str, srt_file_path: Optional[str] = None) -> None:
         """Convert SubStation Alpha v4.0 subtitles to SubRip subtitles.
 
         Arguments:
@@ -194,11 +200,11 @@ class Utils(object):
             srt_file_path {string} -- The path to the SubRip file.
         """
 
-        Utils.__convert_subtitle(ssa_file_path, "ssa", srt_file_path, "srt", "srt")
-        Utils.remove_trailing_newlines(srt_file_path)
+        new_srt_file_path = Utils.__convert_subtitle(ssa_file_path, "ssa", srt_file_path, "srt", "srt")
+        Utils.remove_trailing_newlines(new_srt_file_path)
 
     @staticmethod
-    def srt2microdvd(srt_file_path, microdvd_file_path=None, frame_rate=25.0):
+    def srt2microdvd(srt_file_path: str, microdvd_file_path: Optional[str] = None, frame_rate: float = 25.0):
         """Convert SubRip subtitles to MicroDVD subtitles.
 
         Arguments:
@@ -207,11 +213,11 @@ class Utils(object):
             frame_rate {float} -- The frame rate for frame-based MicroDVD.
         """
 
-        Utils.__convert_subtitle(srt_file_path, "srt", microdvd_file_path, "sub", "microdvd", frame_rate=frame_rate)
-        Utils.remove_trailing_newlines(microdvd_file_path)
+        new_microdvd_file_path = Utils.__convert_subtitle(srt_file_path, "srt", microdvd_file_path, "sub", "microdvd", frame_rate=frame_rate)
+        Utils.remove_trailing_newlines(new_microdvd_file_path)
 
     @staticmethod
-    def microdvd2srt(microdvd_file_path, srt_file_path=None):
+    def microdvd2srt(microdvd_file_path: str, srt_file_path: Optional[str] = None) -> None:
         """Convert MicroDVD subtitles to SubRip subtitles.
 
         Arguments:
@@ -219,11 +225,11 @@ class Utils(object):
             srt_file_path {string} -- The path to the SubRip file.
         """
 
-        Utils.__convert_subtitle(microdvd_file_path, "sub", srt_file_path, "srt", "srt")
-        Utils.remove_trailing_newlines(srt_file_path)
+        new_srt_file_path = Utils.__convert_subtitle(microdvd_file_path, "sub", srt_file_path, "srt", "srt")
+        Utils.remove_trailing_newlines(new_srt_file_path)
 
     @staticmethod
-    def srt2mpl2(srt_file_path, mpl2_file_path=None):
+    def srt2mpl2(srt_file_path: str, mpl2_file_path: Optional[str] = None) -> None:
         """Convert SubRip subtitles to MPL2 subtitles.
 
         Arguments:
@@ -231,11 +237,11 @@ class Utils(object):
             mpl2_file_path {string} -- The path to the MPL2 file.
         """
 
-        Utils.__convert_subtitle(srt_file_path, "srt", mpl2_file_path, "txt", "mpl2")
-        Utils.remove_trailing_newlines(mpl2_file_path)
+        new_mpl2_file_path = Utils.__convert_subtitle(srt_file_path, "srt", mpl2_file_path, "txt", "mpl2")
+        Utils.remove_trailing_newlines(new_mpl2_file_path)
 
     @staticmethod
-    def mpl22srt(mpl2_file_path, srt_file_path=None):
+    def mpl22srt(mpl2_file_path: str, srt_file_path: Optional[str] = None) -> None:
         """Convert MPL2 subtitles to SubRip subtitles.
 
         Arguments:
@@ -243,11 +249,11 @@ class Utils(object):
             srt_file_path {string} -- The path to the SubRip file.
         """
 
-        Utils.__convert_subtitle(mpl2_file_path, "txt", srt_file_path, "srt", "srt")
-        Utils.remove_trailing_newlines(srt_file_path)
+        new_srt_file_path = Utils.__convert_subtitle(mpl2_file_path, "txt", srt_file_path, "srt", "srt")
+        Utils.remove_trailing_newlines(new_srt_file_path)
 
     @staticmethod
-    def srt2tmp(srt_file_path, tmp_file_path=None):
+    def srt2tmp(srt_file_path: str, tmp_file_path: Optional[str] = None) -> None:
         """Convert SubRip subtitles to TMP subtitles.
 
         Arguments:
@@ -255,11 +261,11 @@ class Utils(object):
             tmp_file_path {string} -- The path to the TMP file.
         """
 
-        Utils.__convert_subtitle(srt_file_path, "srt", tmp_file_path, "tmp", "tmp")
-        Utils.remove_trailing_newlines(tmp_file_path)
+        new_tmp_file_path = Utils.__convert_subtitle(srt_file_path, "srt", tmp_file_path, "tmp", "tmp")
+        Utils.remove_trailing_newlines(new_tmp_file_path)
 
     @staticmethod
-    def tmp2srt(tmp_file_path, srt_file_path=None):
+    def tmp2srt(tmp_file_path: str, srt_file_path: Optional[str] = None) -> None:
         """Convert TMP subtitles to SubRip subtitles.
 
         Arguments:
@@ -267,11 +273,11 @@ class Utils(object):
             tmp_file_path {string} -- The path to the SubRip file.
         """
 
-        Utils.__convert_subtitle(tmp_file_path, "tmp", srt_file_path, "srt", "srt")
-        Utils.remove_trailing_newlines(srt_file_path)
+        new_srt_file_path = Utils.__convert_subtitle(tmp_file_path, "tmp", srt_file_path, "srt", "srt")
+        Utils.remove_trailing_newlines(new_srt_file_path)
 
     @staticmethod
-    def suppress_lib_logs():
+    def suppress_lib_logs() -> None:
         import os
         import logging
         os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"
@@ -279,7 +285,7 @@ class Utils(object):
         logging.getLogger("tensorflow").disabled = True
 
     @staticmethod
-    def remove_trailing_newlines(source_file_path, target_file_path=None):
+    def remove_trailing_newlines(source_file_path: str, target_file_path: Optional[str] = None) -> None:
         with open(source_file_path, "r", encoding="utf8") as file:
             content = file.read()
         if target_file_path is not None:
@@ -290,11 +296,18 @@ class Utils(object):
                 file.write(content.rstrip())
 
     @staticmethod
-    def __convert_subtitle(source_file_path, source_ext, target_file_path, target_ext, format, frame_rate=None):
+    def download_file(remote_file_url: str, local_file_path: str) -> str:
+        r = requests.get(remote_file_url, verify=True, stream=True)
+        r.raw.decode_content = True
+        with open(local_file_path, "wb") as file:
+            shutil.copyfileobj(r.raw, file)
+
+    @staticmethod
+    def __convert_subtitle(source_file_path: str, source_ext: str, target_file_path: Optional[str], target_ext: str, format: str, frame_rate: Optional[float] = None) -> str:
         subs = pysubs2.load(source_file_path, encoding="utf-8")
-        if target_file_path is None:
-            target_file_path = source_file_path.replace(".%s" % source_ext, ".%s" % target_ext)
+        new_target_file_path = source_file_path.replace(".%s" % source_ext, ".%s" % target_ext) if target_file_path is None else target_file_path
         if frame_rate is None:
-            subs.save(target_file_path, encoding="utf-8", format_=format)
+            subs.save(new_target_file_path, encoding="utf-8", format_=format)
         else:
-            subs.save(target_file_path, encoding="utf-8", format_=format, fps=frame_rate)
+            subs.save(new_target_file_path, encoding="utf-8", format_=format, fps=frame_rate)
+        return new_target_file_path
